/**
*	WARNING: Do Not Remove This Section
*
*	$RCSfile: simulation_common.h,v $
*	$Revision: 1.32 $
*	$Date: 2008/06/02 16:29:07 $
*	$Author: i.zapreev $
*
*	MRMC is a model checker for discrete-time and continuous-time Markov
*	reward models. It supports reward extensions of PCTL and CSL (PRCTL 
*	and CSRL), and allows for the automated verification of properties
*	concerning long-run and instantaneous rewards as well as cumulative
*	rewards.
*	
* 	Copyright (C) The University of Twente, 2004-2008.
* 	Copyright (C) RWTH Aachen, 2008-2009.
*	Authors: Ivan Zapreev
*
*	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*	Main contact:
*		Lehrstuhl für Informatik 2, RWTH Aachen University
*		Ahornstrasse 55, 52074 Aachen, Germany
*		E-mail: info@mrmc-tool.org
*	
*	Old ontact:
*		Formal Methods and Tools Group, University of Twente,
*		P.O. Box 217, 7500 AE Enschede, The Netherlands,
*		Phone: +31 53 4893767, Fax: +31 53 4893247,
*		E-mail: mrmc@cs.utwente.nl
*
*	Source description: This is a header file for the simulation engine
*	here we intend to define the common functions and data structures.
*/

#include <stdarg.h>

#include "macro.h"

#include "sample_vec.h"
#include "sparse.h"
#include "bitset.h"

#ifndef SIMULATION_COMMON_H

#define SIMULATION_COMMON_H
	
	
	/****************************************************************************/
	/********THE COMMON FUNCTION FOR INVOCATION THE SIMULATION PROCEDURE ********/
	/****************************************************************************/
	
	/**
	* This is a template method for all the method that model check one single initial state
	* @param pStateSpace the state-space matrix
	* @param initial_state the initial state for simulations
	* @param indiff_width the width of the indifference region
	* @param confidence the desired confidence of the answer
	* @param comparator the comparator, one of: 
	*	COMPARATOR_SF_GREATER, COMPARATOR_SF_GREATER_OR_EQUAL,
	*	COMPARATOR_SF_LESS, COMPARATOR_SF_LESS_OR_EQUAL
	* @param prob_bound the probability bound
	* @param arr_index indicates the index in arrays pCiLeftBorders and pCiRightBorders
	*			that corresponds to the initial_state
	*
	* NOTE: The parameters below will contain the return values
	*
	* @param pCiLeftBorders for storing the left conf. int. border
	* @param pCiRightBorders for storing the right conf. int. border
	* @param pYesBitSet contains curr_state if it satisfies the formula
	* @param pNoBitSet contains curr_state if it does not satisfy the formula
	* @param pNumUsedObserv the return-value variable for storing the number of used observations
	* @param args the extra arguments needed for a specific model-checking procedure
	*/
	
	typedef void ( * TPFunctMCOneState ) ( const sparse* pStateSpace, const int initial_state, const double indiff_width,
						const double confidence, const int comparator, const double prob_bound,
						const int arr_index, double *pCiLeftBorders, double *pCiRightBorders,
						bitset * pYesBitSet, bitset * pNoBitSet, unsigned int *pNumUsedObserv, va_list args );
	
	/**
	* This function is responsible for invoking the model procedure either
	* for one or for all initial states the invoked finction for model checking
	* one initial state is passed as an argument pFMCOneState
	* @param pStateSpace the state-space matrix
	* @param confidence this is is confidence we should use for simulations.
	*	Note that since the formulas can be nested one can not use the
	*	overall confidence set in simulation.h
	* @param comparator the comparator, one of: 
	*	COMPARATOR_SF_GREATER, COMPARATOR_SF_GREATER_OR_EQUAL,
	*	COMPARATOR_SF_LESS, COMPARATOR_SF_LESS_OR_EQUAL
	* @param prob_bound the probability bound
	* @param isSimOneInitState TRUE if we simulate one initial state only
	* @param initial_state the initial state index if isSimOneInitState == TRUE
	*
	* NOTE: Below we have pointers to the pointers, these are going to be the return values
	*
	* @param ppYesBitSet this bitset is going to be filled with the states which
	*			satisfy the formula
	* @param ppNoBitSet this bitset is going to be filled with the states which
	*			do not satisfy the formula
	* @param ppCiLeftBorders will contain the left conf. int. borders
	* @param ppCiRightBorders will contain the right conf. int. borders
	* @param pResultSize the pointer to the size of arrays *ppCiLeftBorders and *ppCiRightBorders
	* @param pValidInitialStates the bitset containing all allowed initial
	* @param pMaxNumUsedObserv the return-value variable that will store the maximum number
	* 			(over all initial states) of used observations
	* @param pFMCOneState the pointer to a function which model checks one initial state
	* WARNING: The parameter pFMCOneState should be the last one before "..."
	* @param ... the extra arguments needed for a specific model-checking procedure
	*/
	void modelCheckStatesCommon( const sparse * pStateSpace, const double confidence, bitset ** ppYesBitSet,
					bitset ** ppNoBitSet, double ** ppCiLeftBorders, double ** ppCiRightBorders,
					int * pResultSize, const int comparator, const double prob_bound,
					const int initial_state, const BOOL isSimOneInitState,
					const bitset * pValidInitialStates, unsigned int *pMaxNumUsedObserv,
					TPFunctMCOneState pFMCOneState, ... );
	
	/****************************************************************************/
	/********THE COMMON FUNCTION FOR SIMULATING UNBOUNDED UNTIL CTMC/DTMC********/
	/****************************************************************************/
	
	/**
	* Model checks the unbounded-until operator for one initial state only.
	* Note that this is a univesral procedure that assumes that we work with the embedded DTMC.
	* WARNING: We simulate the states ASSUMING there are no self loops!
	* @param pStateSpace the sparse matrix of the embedded DTMC with good and bad
	*			states made absorbing
	* @param initial_state the initial state for simulations
	* @param indiff_width the width of the indifference region
	* @param confidence the desired confidence of the answer
	* @param comparator the comparator, one of: 
	*	COMPARATOR_SF_GREATER, COMPARATOR_SF_GREATER_OR_EQUAL,
	*	COMPARATOR_SF_LESS, COMPARATOR_SF_LESS_OR_EQUAL
	* @param prob_bound the probability bound
	* @param arr_index indicates the index in arrays pCiLeftBorders and pCiRightBorders
	*			that corresponds to the initial_state
	*
	* NOTE: The parameters below will contain the return values
	*
	* @param pCiLeftBorders for storing the left conf. int. border
	* @param pCiRightBorders for storing the right conf. int. border
	* @param pYesBitSet contains curr_state if it satisfies the formula
	* @param pNoBitSet contains curr_state if it does not satisfy the formula
	* @param pNumUsedObserv the pointer to the return-value variable that will store the
	*		number of observations used for simulating this one state.
	* @param pTransAndGoodStatesArguments contain the two bitsets in the following order:
	*		1. pTransientStates the bitsets containing all the transient states
	*		2. pGoodStates the bitsets containing all the good absorbing states
	*/
	void modelCheckOneStateUUCommon( const sparse* pStateSpace, const int initial_state, const double indiff_width,
						const double confidence, const int comparator, const double prob_bound,
						const int arr_index, double *pCiLeftBorders, double *pCiRightBorders,
						bitset * pYesBitSet, bitset * pNoBitSet, unsigned int *pNumUsedObserv,
						va_list pTransAndGoodStatesArguments );
	
#endif
